import os
import sys
from clint.textui import puts, colored

tests_path = os.path.join(os.path.split(os.path.realpath(sys.argv[0]))[0], "tests")
if not(os.path.exists(tests_path)):
    puts(colored.red(f"Path {tests_path} doesn't exist."))
    exit(0)

existing_tests = []
test_names = list(map(lambda x: x[:x.find('.')], filter(lambda x: x[x.find('.') + 1:] == 'dat', os.listdir(tests_path))))

class save_test:
    def __init__(self, data, ans, new=1):
        self.data = str(data)
        self.ans = str(ans)
        if new:
            self.do_it()
        else:
            existing_tests.append(self)

    def __eq__(self, other):
        if isinstance(other, save_test):
            return self.data == other.data and self.ans == other.ans
        return False

    def do_it(self):
        if self not in existing_tests:
            existing_tests.append(self)
            for l in range(3, 10):
                for i in range(1, 1000):
                    name = (f"%{l}d" % i).replace(" ", "0")
                    if name not in test_names:
                        with open(tests_path + f"/{name}.dat", "w") as file:
                            file.write(self.data)
                        with open(tests_path + f"/{name}.ans", "w") as file:
                            file.write(self.ans)
                        test_names.append(name)
                        return 0


def read_tests():
    for input in test_names:
        with open(tests_path + f"/{input}.dat", "r") as file:
            data = file.read()
        with open(tests_path + f"/{input}.ans", "r") as file:
            ans = file.read()
        save_test(data, ans, 0)
read_tests()



from defined.post_init import GET_CAOS_FOLDER
from defined.generator import generator_text
from defined.constants import sync_info
from offline.modify import set_tasks_dir
from utils import get_problems, c_file

import os
from bs4 import BeautifulSoup as bs
from clint.textui import puts


def sync(session, sync_samples, sync_statements, sync_other, target_contest='all', extension='.c'):
    if sync_other:
        puts(sync_info)
        exit(0)
    tasks_dir_path = GET_CAOS_FOLDER()
    if tasks_dir_path == "-":
        set_tasks_dir()
        tasks_dir_path = GET_CAOS_FOLDER()

    files = os.listdir(tasks_dir_path)
    problems = get_problems(session)

    for problem in problems:
        short_name = problem['short name']
        contest = short_name[:short_name.find('-')]
        if target_contest != 'all' and contest != target_contest:
            continue

        contest_dir = os.path.join(tasks_dir_path, contest)

        if contest not in files:
            os.mkdir(contest_dir)
            files.append(contest)

        num = short_name[short_name.find('-') + 1:]
        task_dir = os.path.join(contest_dir, num)
        if num not in os.listdir(contest_dir) :
            os.mkdir(task_dir)
        c_file_name = c_file(os.listdir(task_dir))
        if not(c_file_name):
            c_file_name = problem["name"].replace("/", "_") + extension
            open(os.path.join(task_dir, c_file_name), 'w').close()
            with open(os.path.join(task_dir, 'gen.py'), 'a') as generator:
                if os.stat(os.path.join(task_dir, 'gen.py')).st_size < 10:
                    generator.write(generator_text)

            if 'tests' not in os.listdir(task_dir) :
                os.mkdir(task_dir + '/tests')
        if sync_statements or sync_samples:
            task_html = session.get(problem['href'])

            if sync_statements:
                if 'statement.txt' not in os.listdir(task_dir):
                    soup = bs(task_html.content, 'html.parser')

                    start_text = f'Problem {short_name}: {problem["name"]}'
                    finish = soup.text.find('Examples')
                    if finish == -1:
                        finish = soup.text.find('\nSubmit a solution\n\n')
                    statement = soup.text[soup.text.find(start_text) + len(start_text) + 1: finish]

                    with open(os.path.join(task_dir, 'statement.txt'), 'w') as statementfile:
                        statementfile.write(statement)

            if sync_samples:
                soup = bs(task_html.content, 'html.parser')

                if 'Input' in soup.text and 'Output' in soup.text:
                    sample_input = soup.text[soup.text.find('Input') + 6: soup.text.find('Output')]
                    sample_output = (
                        soup.text[soup.text.find('Output') + 7: soup.text.find('\nSubmit a solution\n\n')] + '\n')

                    filein_name = os.path.join(task_dir, 'tests', '000.dat')

                    fileout_name = os.path.join(task_dir, 'tests', '000.ans')

                    with open(filein_name, 'w') as filein:
                        filein.write(sample_input)
                    with open(fileout_name, 'w') as fileout:
                        fileout.write(sample_output)



from defined.post_init import GET_COMPILER, GET_CAOS_FOLDER

from .modify import set_tasks_dir
from utils import c_file
from clint.textui import puts, colored, indent
import os

abort_text = colored.red("Start with a folder with .c file and tests folder or provide both -c and -t flags for testing")

def before_test(args):
    grouped = args.grouped

    debug_mode = []
    if '-d' in grouped:
        if len(grouped['-d']) == 0:
            debug_mode = ['000']
        else:
            debug_mode = grouped['-d'].all

    if '-c' not in grouped or '-t' not in grouped or len(grouped['-c']) == 0 or len(grouped['-t']) == 0:
        task_path = os.getcwd()
    else:
        #not best solution cause have to add if everywhere and import in a lot of places, but i dont have a better idea withour ruining of order of iimport
        tasks_dir_path = GET_CAOS_FOLDER()
        if tasks_dir_path == "-":
            set_tasks_dir()
            tasks_dir_path = GET_CAOS_FOLDER()
        task_path = os.path.join(tasks_dir_path, grouped['-c'][0], grouped['-t'][0])

    tests_path = os.path.join(task_path, 'tests')
    c_file_name = c_file(os.listdir(task_path))

    if not(c_file_name):
        puts(colored.red(f"No .c file found in {task_path}. ") + abort_text)
        exit(0)

    if not(os.path.exists(tests_path)):
        puts(colored.red(f"Path {tests_path} doesn't exist. ") + abort_text)
        exit(0)

    test(task_path, tests_path, c_file_name, debug_mode)

def test(task_path, tests_path, c_file_name, debug_vals = []):

    tests = os.listdir(tests_path)

    if len(debug_vals) > 0:
        inputs = debug_vals
    else:
        inputs = filter(lambda x: x[x.find('.') + 1:] == 'dat', tests)
        inputs = list(map(lambda x: x[:x.find('.')], inputs))

    if not inputs:
        puts(colored.red(f"No tests found at {tests_path}"))
        exit(0)

    if 'a.out' in os.listdir(task_path):
        os.remove(f'{task_path}/a.out')



    os.system(GET_COMPILER().format(os.path.join(task_path, c_file_name.replace(" ", "\ ") )) + f" -o {task_path}/a.out")
    if 'a.out' not in os.listdir(task_path):
        puts(colored.red(f"Compilation error, aborted"))
        exit(0)

    for input in inputs:
        if input + '.ans' not in tests:
            puts(colored.yellow(f"No matching output for test {input}.dat. Skip it."))
            continue

        run_test(input, task_path, len(debug_vals) > 0)

    os.remove(f'{task_path}/temp')
    os.remove(f'{task_path}/a.out')


def run_test(test, task_path, debug_mode):
    input_path = os.path.join(task_path, 'tests', test + '.dat')
    output_path = os.path.join(task_path, 'tests', test + '.ans')

    os.system(f'{task_path}/a.out < {input_path} > {task_path}/temp'.format())

    with open(output_path, 'r') as output_file:
        with open(f'{task_path}/temp', 'r') as temp_file:
            expected_lines = output_file.readlines()
            resulting_lines = temp_file.readlines()

            if expected_lines == resulting_lines:
                puts(colored.green(f"Test {test}: OK!"))
            else:
                puts(colored.red(f"Test {test}: Failed!"))
                if not(debug_mode):
                    find_diff(expected_lines, resulting_lines)
                    exit(0)
                else:
                    puts(colored.yellow("Expected:"))
                    for line in expected_lines:
                        print(line, end = "")
                    puts(colored.yellow("Got output:"))
                    for line in resulting_lines:
                        print(line, end = "")

def find_diff(expected_lines, resulting_lines):
    count = 0
    for (line, (expected, resulting)) in enumerate(zip(expected_lines, resulting_lines)):
        if expected != resulting:
            with indent(quote=f'  Line #{line + 1} > '):
                puts(colored.blue(f"Expected:  {expected}"), newline=False)
                puts(colored.yellow(f"Resulting: {resulting}"))
            count += 1

        if count == 10:
            break

    puts(colored.red(f"Resulting output saved to the 'temp' file."))



from defined.post_init import CONFIG_PATH, GET_CAOS_FOLDER
from defined.constants import CLANG_FORMAT_STYLE_STRING
from offline.modify import set_tasks_dir
from utils import c_file

import configparser
import os
import shutil
from clint.textui import puts, colored, indent
from shutil import which


def style(args):
    tasks_dir_path = GET_CAOS_FOLDER()
    if tasks_dir_path == "-":
        set_tasks_dir()
        tasks_dir_path = GET_CAOS_FOLDER()

    grouped = args.grouped
    if ('-c' not in grouped or '-t' not in grouped or len(grouped['-c']) == 0 or len(grouped['-t']) == 0) and (not '--all' in args.flags):
        puts(colored.red("Provide both -c and -t flags for formating or choose --all"))
        exit(0)

    #this is just getting clang-format bin and sending files there....
    """
    config = configparser.ConfigParser()
    config['Tools'] = {}
    config.read(CONFIG_PATH)
    if '--set-tool' in args.grouped:
        config['Tools']['clang-format'] = args.grouped['--set-tool']
        with open(CONFIG_PATH, 'w') as configfile:
            config.write(configfile)

        puts(colored.green('clang-format tool path changed succesfully!'))
        exit(0)

    tool = args.grouped.get(
        '-format', config['Tools'].get('clang-format',
                                  shutil.which('clang-format')))

    if not tool or not os.path.exists(tool):
        puts(
            colored.red(
                "clang-format is not found in $PATH and not specified explicitly."
                "Try modifying $PATH, or use one of '--set-tool' and '-t' flags."
            ))
        exit(1)
    """
    tool = "clang-format"

    if which(tool) is None:
        puts(colored.red("clang-format is not installed!"))
        puts("try to install it via your packet manager")
        puts("apt-get/yum/brew/..." + colored.yellow(" install clang-format"))
        exit(0)

    files = []
    if '--all' in args.flags:
        for contest in os.listdir(tasks_dir_path):
            for task in os.listdir(os.path.join(tasks_dir_path, contest)):
                task_path = os.path.join(tasks_dir_path, contest, task)
                c_file_name = c_file(os.listdir(task_path))
                files.append(os.path.join(task_path, c_file_name.replace(" ", "\ ")))
    else:
        contest = grouped['-c'][0]
        task = grouped['-t'][0]
        task_path = os.path.join(tasks_dir_path, contest, task)
        c_file_name = c_file(os.listdir(task_path))
        files.append(os.path.join(task_path, c_file_name.replace(" ", "\ ")))

    command = (f"{tool} "
               f"-style={CLANG_FORMAT_STYLE_STRING} "
               "--verbose -i "
               f"{' '.join(str(file) for file in files)}")
    os.system(command)



import os
from clint.textui import puts, colored

def run_generator():

    generator = os.path.join(os.getcwd(), "gen.py")
    if not(os.path.isfile(generator)):
        puts(colored.red("start with a gen.py in your folder"))
        exit(0)
    os.system("python3 gen.py")



from clint.textui import puts, colored
import os

options = ['test', 'args', 'folder', 'help', 'gen']

def handler(mode, args):
    from .test import before_test
    from .modify import set_gcc_args, set_tasks_dir
    from .help import print_help
    from .generate import run_generator

    if mode == 'test':
        before_test(args)
    elif mode == 'gen':
        run_generator()
    elif mode == 'args':
        set_gcc_args()
    elif mode == 'folder':
        set_tasks_dir()
    elif mode == 'help':
        print_help()

    exit(0)
